using System;
using System.Drawing;
using System.Windows.Forms;

namespace HeuristicLab.Visualization {
  public class ZoomListener : IMouseEventListener {
    private readonly Point startPoint;

    public event RectangleHandler DrawRectangle;
    public event RectangleHandler SetClippingArea;

    public ZoomListener(Point startPoint) {
      this.startPoint = startPoint;
    }

    #region IMouseEventListener Members

    public void MouseMove(object sender, MouseEventArgs e) {
      if(DrawRectangle != null) {
        DrawRectangle(CalcRectangle(e.Location));
      }
    }

    public void MouseUp(object sender, MouseEventArgs e) {
     if(SetClippingArea != null) {
       SetClippingArea(CalcRectangle(e.Location));
     }
    }

    #endregion

    private Rectangle CalcRectangle(Point actualPoint) {
      int x = Math.Min(startPoint.X, actualPoint.X);
      int y = Math.Min(startPoint.Y, actualPoint.Y);
      int width = Math.Abs(actualPoint.X - startPoint.X);
      int height = Math.Abs(actualPoint.Y - startPoint.Y);
      
      return new Rectangle(x, y, width, height);
    }

    public static RectangleD ZoomClippingArea(RectangleD clippingArea, double zoomFactor) {
      double x1, x2, y1, y2, width, height;

      width = clippingArea.Width * zoomFactor;
      height = clippingArea.Height * zoomFactor;

      x1 = clippingArea.X1 - (width - clippingArea.Width) / 2;
      y1 = clippingArea.Y1 - (height - clippingArea.Height) / 2;
      x2 = width + x1;
      y2 = height + y1;

      return new RectangleD(x1, y1, x2, y2);
    }
  }
}